#include "AIMNetThread.h"

//---------------------------------------------------------------------

void AIMNetThread::SendPacket( DataContainer packet, char channel ) {

	DataContainer sendData;
	
	// tack on the FLAP header
	sendData << '\x2a';
	sendData << channel;
	sendData << ToAIMWord( sendSequence );
	sendData << ToAIMWord( packet.length() );
	
	// increment the sequence number
	if( sendSequence == 0xFFFF )
		sendSequence = 0;
	else
		++sendSequence;	

	// Now add the packet itself
	sendData << packet;

	// bon voyage...
	Transmit( sendData );
}

//---------------------------------------------------------------------

void AIMNetThread::SendSNACPacket( SNAC_Object& obj ) {

	// SNAC's are always send on channel 0x02
	SendPacket( obj.Packet(), 0x02 );
}

//---------------------------------------------------------------------

void AIMNetThread::Transmit( DataContainer packet ) {

	// dump the packet
	printf( "----> Sending a packet!\n" );
	for( short i = 0; i < packet.length(); ++i ) {
		printf( "%d%c", (int)packet[i], char(9) );
		if( !((i+1) % 8) )
			printf( "\n" );
	}
	printf( "\nsehr done.\n\n" );

	//socket_send( packet.c_ptr(), packet.length() );
	SendData( packet.c_ptr(), packet.length() );
}

//---------------------------------------------------------------------

char AIMNetThread::Receive() {

	int recvCount;
	char header[3] = "";
	char channel;
	unsigned short seq, flapLen;
	char* dataTemp = NULL;

	// get 2 byte FLAP_id, presumably 0x2aXX
	recvBuf = "";
	recvCount = ReadData( header, 2 );
	if( recvCount == -1 || header[0] != 0x2a )
		return 0;
	channel = header[1];	
	printf( "start data: %c %c\n", header[0], header[1] );
	
	// get the server's sequence number
	recvCount = ReadData( &seq, 2 );
	seq = GetWord( (char*)&seq );
	printf( "sequence number: %u\n", seq );
	
	// check the sequence numbers
	if( !recvSequence )
		recvSequence = seq;
	else if( seq != ++recvSequence ) {
	
		// odd... this packet was out of order. Oh well.
		printf( "***WARNING: received packet out of order!\n" );
	}
	
	// Get the FLAP data length (the last 2 bytes of the header)
	recvCount = ReadData( &flapLen, 2 );
	flapLen = GetWord( (char*)&flapLen );
	printf( "FLAP data length: %u\n", flapLen );
	
	// Now, read the rest of the packet
	dataTemp = new char[flapLen];
	recvCount = ReadData( dataTemp, flapLen );
	recvBuf = DataContainer( dataTemp, flapLen );
	
	// return the channel
	return channel;
}

//---------------------------------------------------------------------

void AIMNetThread::SendAppMessage( BMessage* msg ) {

	msg->AddInt32( "proto", (int32)AIM_PROTOCOL );
	
	BMessenger sender( APP_SIGNATURE );
	sender.SendMessage( msg );
	
	delete msg;
}

//---------------------------------------------------------------------