#include "AIMNetThread.h"
#include "AIMConstants.h"
#include "Globals.h"
#include "Say.h"

//---------------------------------------------------------------------

// Handles the login "steps"
void AIMNetThread::DoLogin( BMessage* info ) {

	char channel;

	printf( "dologin()... %d\n", preLoginPhase );

	// do stuff based on the login phase
	switch( preLoginPhase ) {
		
		// Just after the initial connection
		case 1:
			LoginStep( "Connected to authorization server..." );
			sendSequence = 0;
			recvSequence = 0;
			break;

		// Connection acknowledgement received from authorization server
		case 2:
			channel = Receive();
			printf( "ack packet recv'd on channel %c\n", channel );
			SendLoginData();
			break;
		
		// Authorization response received... check to see if we can login
		case 3:
			LoginStep( "Authorization response received..." );
			ParseLoginResponse( recvBuf );
			break;
		
		// Connected to BOS server
		case 4:
			LoginStep( "Connected to AIM server..." );
			printf( "DoLogin(): connected to BOS server\n" );
			break;
		
		// BOS connection acknowledgment... send the sign on request
		case 5:
			channel = Receive();
			printf( "BOS connection ack packet recv'd on channel %d  (size: %d bytes)\n", (int)channel, recvBuf.length() );
			SendBOSRequest();
			break;
		
		// Host Ready response received... send the rate request
		case 6:
			LoginStep( "Receiving Host Ready response...\n" );
			printf( "host ready response received\n" );
			ReceiveBOSResponse();
			SendBOSRequest();
			break;
			
		// Rate response received... send rate acknowledgement (and other stuff)
		case 7:
			LoginStep( "Receiving rate response..." );
			ReceiveBOSResponse();
			SendBOSRequest();
			break;
		
		// Process the responses to various other stuff we have sent up	
		case 8:
		case 9:
		case 10:
			LoginStep( "", false );
			printf( "got something! yeah!\n" );
			ReceiveBOSResponse();
			break;
		
		// server is done sending responses... send up more stuff
		case 11:
			printf( "boo-rah!\n" );
			LoginStep( "Receiving host response..." );
			ReceiveBOSResponse();
			SendBOSRequest();
			break;
		
		// minimum report interval
		case 12:
			LoginStep( "Receiving report interval..." );
			ReceiveBOSResponse();
			break;
			
		// mystery packet (comes at the end of login)
		case 13:
			LoginStep( "Finalizing login..." );
			ReceiveBOSResponse();
			NowOnline();
			return;
	};

	// Time for the next step
	++preLoginPhase;	
}

//---------------------------------------------------------------------

// Login function
void AIMNetThread::Login( BMessage* message ) {
	
	// Host and port
	char aim_host[255];
	int32 aim_port = 0;	
	
	// Set the number of steps in this particular login
	BMessenger sender( APP_SIGNATURE );
	BMessage* sendMessage = new BMessage(BEAIM_SET_LOGIN_STEP_COUNT);
	sendMessage->AddFloat( "count", 29.0 );
	sender.SendMessage( sendMessage );
	
	// Get the host and port from the prefs
	prefs->ReadString( "aim_host", aim_host, "login.oscar.aol.com", 255 );
	aim_port = prefs->ReadInt32( "aim_port", 5190 );
	
	// Since this is all event based, we have to make copies of the sn/pw
	loginScreenName = new char[ strlen((char*)message->FindString("userid"))+1 ];
	strcpy( loginScreenName, (char*)message->FindString("userid") );
	loginPassword = new char[ strlen((char*)message->FindString("password"))+1 ];
	strcpy( loginPassword, (char*)message->FindString("password") );	
	
	// Time to begin... connect to the authorization server
	preLoginPhase = 1;
	LoginStep( "Connecting to authorization server..." );
	//Connect( aim_host, aim_port, true );
	Connect( aim_host, aim_port );
}

//---------------------------------------------------------------------

void AIMNetThread::SendLoginData() {

	DataContainer loginPacket = "";
	DataContainer clientProfile;
	
	// raw data
	loginPacket << ToAIMWord(0x0000);
	loginPacket << ToAIMWord(0x0001);
	
	// TLV's for the screen name and password
	loginPacket << ToTLV( 0x01, strlen(loginScreenName), loginScreenName );
	loginPacket << ToTLV( 0x02, strlen(loginPassword), EncryptPassword(loginPassword) );
	
	// Now, add the client profile
	clientProfile = "BeAIM Copyright (c) 1999 <slarty@legions.com>"
	                "  Beta Version 1: for BeOS R4 on x86 and PPC"
	                "  All Rights Reserved. Contact me for movie deals.";
	loginPacket << ToTLV( 0x03, clientProfile.length(), clientProfile );
	
	// add some more thrilling info
	loginPacket << ToTLV( 0x16, 0x02, ToAIMWord(0x0001) );
	loginPacket << ToTLV( 0x17, 0x02, ToAIMWord(BEAIM_MAJOR_VERSION) );
	loginPacket << ToTLV( 0x18, 0x02, ToAIMWord(BEAIM_MINOR_VERSION) );
	loginPacket << ToTLV( 0x1a, 0x02, ToAIMWord(BEAIM_BUILD) );
	loginPacket << ToTLV( 0x0e, 0x02, "us" );
	loginPacket << ToTLV( 0x0f, 0x02, "en" );
	loginPacket << ToTLV( 0x09, 0x02, ToAIMWord(0x0015) );
	
	// Send off that big bad packet...
	LoginStep( "Sending authorization data..." );
	SendPacket( loginPacket, 0x01 );
	
	// we're done w/ the sn/pw now... delete them
	delete loginScreenName;
	delete loginPassword;
	loginScreenName = loginPassword = NULL;
}

//---------------------------------------------------------------------

DataContainer AIMNetThread::EncryptPassword( char* password )
{
	char encoding_table[] = {
		0xf3, 0xb3, 0x6c, 0x99,
		0x95, 0x3f, 0xac, 0xb6,
		0xc5, 0xfa, 0x6b, 0x63,
		0x69, 0x6c, 0xc3, 0x9f
	};

	DataContainer encoded;
	unsigned int i;
	
	// encode the password
	for (i = 0; i < strlen(password); i++)
		encoded << char( password[i] ^ encoding_table[i] );
		
	return encoded;
}

//---------------------------------------------------------------------

void AIMNetThread::ParseLoginResponse( DataContainer data ) {

	printf( "parsing login response\n" );

	char channel;
	DataContainer nextServer;
	
	// Get the authorization response packet
	channel = Receive();
	if( channel != 0x04 ) {
		printf( "Hmmmm.... channel should have been 0x04, but was %c instead! Bad.\n", channel );
		return;
	}

	// Decode it!
	unsigned short Tag, Len, i = 0;
	unsigned short errNum;

	// Go through all the TLV's in this packet
	while( i < recvBuf.length() ) {

		// Get the Tag, Length, and Value out of recvBuf
		Tag = GetWord( recvBuf, i );	i += 2;
		Len = GetWord( recvBuf, i );	i += 2;
		DataContainer Value( recvBuf.getrange(i,Len) );
		i += Len;
		
		printf( "%%%%%%%%%%%%%% NEW TLV!!!\n" );
		printf( "    Tag: %X\n", Tag );
		printf( "    Len: %X\n\n", Len );

		// Go through the possible tag values
		switch( Tag ) {

			// The "official" screen name
			case 0x01:
				Value << char(0);
				printf( "official screen name: %s\n", Value.c_ptr() );
				break;
			
			// error message URL
			case 0x04:
				Value << char(0);
				printf( "error... go to %s for more info\n", Value.c_ptr() );
				break;
				
			// BOS IP address (redirect)
			case 0x05:
				Value << char(0);
				nextServer = Value;
				printf( "The BOS IP to redirect to is %s\n", Value.c_ptr() );
				break;

			// Received the authorization cookie
			case 0x06:
				cookie = Value;
				printf( "got the authorization cookie. Woohoo!\n" );
				break;

			// email address
			case 0x11:
				Value << char(0);
				printf( "Your email address (according to OSCAR) is %s\n", Value.c_ptr() );
				break;

			// registration status
			case 0x13:
				errNum = (Value[0] << 8) + Value[1];
				printf( "registration status is %u, whatever that means.\n", errNum );
				break;

			// Error number
			case 0x08:
				errNum = (Value[0] << 8) + Value[1];
				switch( errNum ) {

					// Invalid screen name
					case 0x01:
						LoginFailure( "You entered an invalid screen name." );
						printf( "ERROR: invalid screen name!\n" );
						break;
						
					// Invalid password
					case 0x05:
						printf( "ERROR: invalid password!\n" );
						LoginFailure( "You entered an invalid password." );
						break;
						
					// Too many logins
					case 0x18:
						LoginFailure( "You have logged on too many times in a set interval!\nWait a while, and try again." );
						printf( "ERROR: logged on too many times in a set interval!\n" );
						break;

					// something else
					default:
						LoginFailure( "Login failed, for an unknown reason. Sorry." );
						printf( "ERROR: something weird has gone wrong, and we don't know what! Aah!\n" );
						break;
				}
				connected = false;
				break;
				
			// unknown TLV
			default:
				printf( "Hmmmm... no idea what this TLV is. Ignoring.\n" );
				break;
		}
	}
	
	// ahhh... all done with the authorization. Disconnect from this server.
	connected = false;
	recvSequence = 0;

	//Disconnect();
	Close();
	
	// Now, we have the cookie and the next server, so attempt to connect to that.
	if( preLoginPhase ) {
		LoginStep( "Connecting to AIM server..." );	
		Connect( nextServer.c_ptr(), prefs->ReadInt32("aim_port", 5190) );
	}
}

//---------------------------------------------------------------------

void AIMNetThread::SendBOSRequest() {

	DataContainer sendBuf;
	SNAC_Object snac;

	// The BOS Sign On Command... basically just resend the cookie
	if( preLoginPhase == 5 ) {
		sendBuf << ToAIMWord( 0x0000 );
		sendBuf << ToAIMWord( 0x0001 );
		sendBuf << ToTLV( 0x06, cookie.length(), cookie );
		LoginStep( "Sending login data..." );
		SendPacket( sendBuf, 0x01 );
		return;
	}
	
	// Send a rate request, to find out how fast we can send SNAC's
	if( preLoginPhase == 6 ) {
		printf( "// Send a rate request, to find out how fast we can send SNAC's\n ");
		snac.Update( 0x01, 0x06, 0, 0, 0x1015 );
		LoginStep( "Sending rate request..." );
		SendSNACPacket( snac );
		return;
	}
	
	// Send lots of stuff requests... order doesn't matter here, so
	// we can "chunk" them together
	if( preLoginPhase == 7 ) {

		LoginStep( "Sending information requests..." );
		
		// Send an acknowledgement that we got the rate response
		snac.Update( 0x01, 0x08, 0x00, 0x00, 0x1016 );
		snac.data << ToAIMWord( 0x0001 ) << ToAIMWord( 0x0002 )
				  << ToAIMWord( 0x0003 ) << ToAIMWord( 0x0004 );
		SendSNACPacket( snac );
		LoginStep( "", false );

		// Set privacy flags
		snac.Update( 0x01, 0x14, 0x00, 0x00, 0x1017 );
		snac.data << ToAIMWord( 0x0000 ) << ToAIMWord( 0x0003 );
		SendSNACPacket( snac );
		LoginStep( "", false );

		// Send request for our user info
		printf( "// Send request for our user info\n" );
		SendSNACPacket( snac );
		LoginStep( "", false );
		
		// Send BOS rights request
		snac.Update( 0x09, 0x02, 0x00, 0x00, 0x101a );
		SendSNACPacket( snac );
		LoginStep( "", false );
		
		// Send BuddyList rights request
		snac.Update( 0x03, 0x02, 0x00, 0x00, 0x101b );
		SendSNACPacket( snac );
		LoginStep( "", false );
		
		// Send locate service rights request
		snac.Update( 0x02, 0x02, 0x00, 0x00, 0x101c );
		SendSNACPacket( snac );
		LoginStep( "", false );
		
		// Send IM Parameter info request
		snac.Update( 0x04, 0x04, 0x00, 0x00, 0x101d );
		SendSNACPacket( snac );
		LoginStep( "Receiving information responses..." );
		
		return;
	}
	
	// send up another big load of stuff
	if( preLoginPhase == 11 ) {

		LoginStep( "Sending login data..." );

		// Optional(?) SNAC number 1
		snac.Update( 0x09, 0x04, 0x00, 0x00, 0x101e );
		snac.data << ToAIMWord( 0x00 ) << ToAIMWord( 0x001f );
		SendSNACPacket( snac );

		// Optional(?) SNAC number 2
		snac.Update( 0x09, 0x07, 0x00, 0x00, 0x101f );
		SendSNACPacket( snac );
		
		// Send up the buddy list
		DataContainer BList;
		LoadBuddyList( BList );
		LoginStep( "Sending buddy list..." );
		snac.Update( 0x03, 0x04, 0x00, 0x00, 0x1020 );
		snac.data << BList;
		SendSNACPacket( snac );
		
		char noProfileString[512];
		strcpy( noProfileString, "This user has a version of BeAIM that doesn't support editing their own profile. For the latest version, check out BeAIM's entry on BeBits <a href=\"http://www.bebits.com/app.php3/1/\">here</a>, or go to the BeAIM home page <a href=\"http://www.fifthace.com/beaim\">here</a>." );
		
		// Send up the user profile
		LoginStep( "Sending user profile..." );
		snac.Update( 0x02, 0x04, 0x00, 0x00, 0x1021 );
		snac.data << ToTLV( 0x01, 0x0d, "text/x-aolrtf" )
				  << ToTLV( 0x02, strlen(noProfileString), noProfileString );
		SendSNACPacket( snac );

		// send set initial ICBM parameter
		snac.Update( 0x04, 0x02, 0x00, 0x00, 0x1022 );
		snac.data << ToAIMWord(0x01)   << ToAIMWord(0x00)   << ToAIMWord(0x03)
				  << ToAIMWord(0x1f3f) << ToAIMWord(0x03e7) << ToAIMWord(0x03e7)
				  << ToAIMWord(0x00)   << ToAIMWord(0x64);
		SendSNACPacket( snac );
		
		// send client online/ready
		LoginStep( "Sending Client Ready..." );
		snac.Update( 0x01, 0x02, 0x00, 0x00, 0x1023 );
		snac.data << ToAIMWord(0x01) << ToAIMWord(0x02) << ToAIMWord(0x01)
				  << ToAIMWord(0x13) << ToAIMWord(0x09) << ToAIMWord(0x01)
				  << ToAIMWord(0x01) << ToAIMWord(0x01) << ToAIMWord(0x03)
				  << ToAIMWord(0x01) << ToAIMWord(0x01) << ToAIMWord(0x01)
				  << ToAIMWord(0x04) << ToAIMWord(0x01) << ToAIMWord(0x01)
				  << ToAIMWord(0x01) << ToAIMWord(0x02) << ToAIMWord(0x01)
				  << ToAIMWord(0x01) << ToAIMWord(0x01) << ToAIMWord(0x08)
				  << ToAIMWord(0x01) << ToAIMWord(0x01) << ToAIMWord(0x01)
				  << ToAIMWord(0x06) << ToAIMWord(0x01) << ToAIMWord(0x01)
				  << ToAIMWord(0x01) << ToAIMWord(0x0a) << ToAIMWord(0x01)
				  << ToAIMWord(0x01) << ToAIMWord(0x01) << ToAIMWord(0x0b)
				  << ToAIMWord(0x01) << ToAIMWord(0x01) << ToAIMWord(0x01);
		SendSNACPacket( snac );
		
		// ... and we're almost officially online!
		return;
	}
}

//---------------------------------------------------------------------

void AIMNetThread::ReceiveBOSResponse() {

	// Get the actual data
	char channel = Receive();
	SNAC_Object snac;
	
	// The Host-Ready response
	if( preLoginPhase == 6 || preLoginPhase == 7 ) {
		if( channel != 0x02 ) {
			printf( "received response on the wrong channel (%c instead of 0x02)! bad.\n", channel );
			return;
		}
		snac = SNAC_Object(recvBuf);
		DecodeSNACPacket( snac );
		return;
	}
	
	// the "default" case
	snac = SNAC_Object( recvBuf );
	DecodeSNACPacket( snac );
}

//---------------------------------------------------------------------

// woo-hoo! We're all logged in now. Tell the app.
void AIMNetThread::NowOnline() {

	preLoginPhase = 0;
	printf( "You are now online... officially!\n" );
	
	// Send a message saying that the signon was successful
	BMessage* goodSignOn = new BMessage(BEAIM_SIGN_ON_SUCCESSFUL);
	SendAppMessage( goodSignOn );
}

//---------------------------------------------------------------------

// Whoops... something went wrong. Cancel the login.
void AIMNetThread::LoginFailure( char* reason ) {

	BMessage* sendMessage;

	// then send a message saying that the login has failed, with a reason.
	sendMessage = new BMessage( BEAIM_LOGIN_FAILURE );
	sendMessage->AddString( "reason", reason );
	SendAppMessage( sendMessage );

	// first, do a bit of cleanup in here.
	connected = false;
	//Disconnect();
	Close();
	preLoginPhase = 0;
}

//---------------------------------------------------------------------