#include <Alert.h>
#include <string.h>
#include <stdio.h>
#include "BeAIM.h"
#include "constants.h"
#include "Globals.h"
#include "AIMNetThread.h"
#include "TrayIcon.h"
#include "Say.h"

//---------------------------------------------------------------------

main( int argc, void** argv )
{	
	strcpy( AppFileName, (char*)argv[0] );

	BeAIMApplication myApplication;
	myApplication.Run();
	return(0);
}

//---------------------------------------------------------------------

BeAIMApplication::BeAIMApplication()
		  		  : BApplication(APP_SIGNATURE)
{
	// Do startup stuff
	DoBeAIMStartupStuff();

	// create the signon window and show it
	windows->OpenSignOnWindow();
	
	// Fake Incoming Message...
	/*
	comm->SetProgramUser( "MegaPiffle" );
	windows->OpenBuddyList();
	BMessage msgRecv(BEAIM_INCOMING_IM);
	msgRecv.AddString( "userid", "BigZaphod" );
	msgRecv.AddInt32( "wtype", 1 );
	msgRecv.AddInt32( "idletime", 0 );
	msgRecv.AddInt32( "proto", int32(AIM_PROTOCOL) );
	msgRecv.AddString( "message", "<font color=\"#BB00BB\" size=5>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!<br>Test!</font>" );
	//msgRecv.AddString( "message", "Embedding a <a href=\"http://www.yahoo.com\">Link</a><a href=\"http://www.amazon.com\"> in the text</a>... <a href=\"http://www.puddle.org\">oh yeah!</a>" );
	//msgRecv.AddString( "message", "This<b>is<i>a</i>test</b>of the most importance!" );
	//msgRecv.AddString( "message", "This<b>is<i>a</i>test</b>of something important" );
	windows->ForwardIncomingMessage( &msgRecv );
	*/
	
	//Fake Oncoming Buddy...
	/*
	comm->SetProgramUser( "MegaPiffle" );
	windows->OpenBuddyList();
	BMessage msgRecv(BEAIM_ONCOMING_BUDDY);
	msgRecv.AddString( "userid", "BigZaphod" );
	msgRecv.AddInt32( "idletime", 0 );
	msgRecv.AddInt32( "proto", int32(AIM_PROTOCOL) );
	windows->SendBuddyListMessage( &msgRecv );
	*/

}

//---------------------------------------------------------------------

BeAIMApplication::~BeAIMApplication() {

}

//---------------------------------------------------------------------

bool BeAIMApplication::QuitRequested() {

	DoBeAIMShutdownStuff();
	return true;
}

//---------------------------------------------------------------------

// Central message "switchboard" of BeAIM
void BeAIMApplication::MessageReceived( BMessage* message )
{
	message->AddInt32( "proto", int32(AIM_PROTOCOL) );

	switch(message->what)
	{
		// forward these to the login window
		case BEAIM_SET_LOGIN_STEP_COUNT:
		case BEAIM_LOGIN_STEP:
		case BEAIM_LOGIN_FAILURE:
			windows->SendSignOnMessage( message );
			break;

		case BEAIM_ADD_PERSON:
		case BEAIM_ONCOMING_BUDDY:
		case BEAIM_OFFGOING_BUDDY:
			windows->SendBuddyListMessage( message );
			break;

		case BEAIM_SEND_MESSAGE:
			sounds->PlaySound( WS_MSGSEND );
		case BEAIM_CANCEL_SIGN_ON:
		case BEAIM_GET_PERSON_INFO:
		case BEAIM_SIGN_ON:
		case BEAIM_PEOPLELIST_COMMIT:
		case BEAIM_SEARCH_BY_EMAIL:
		case BEAIM_NOW_IDLE:
		case BEAIM_NOW_ACTIVE:
		case BEAIM_SEND_IDLE_PULSE:
			net->PostMessage( message );
			break;
			
		case BEAIM_SEND_ARBITRARY_DATA:
			windows->MakeDataSenderWindow();
			break;

		case BEAIM_OPEN_IM_WINDOW:
		case BEAIM_OPEN_USER_WINDOW:
			windows->OpenUserWindow( message );
			break;

		case BEAIM_IM_WINDOW_OPENED:
			windows->UserWindowOpened( message );
			break;

		case BEAIM_IM_WINDOW_CLOSED:
			windows->UserWindowClosed( message );
			break;

		case BEAIM_TOGGLE_HIDDEN:
			windows->ToggleHidden();
			break;

		case BEAIM_SIGN_ON_SUCCESSFUL:
			LoggedIn = true;
			windows->OpenBuddyList();
			windows->CloseSignOnWindow();
			break;

		case BEAIM_OPEN_PREFS:
			windows->OpenPrefsWindow();
			break;
			
		case BEAIM_UPDATE_INFO:
		case BEAIM_INCOMING_IM:
			windows->ForwardIncomingMessage( message );
			break;
			
		case BEAIM_OPEN_EMAIL_SEARCH:
			windows->OpenEmailSearchResultsWindow( message );
			break;
			
		case BEAIM_EMAIL_SEARCH_RESULTS:
			windows->ForwardEmailSearchResults( message );
			break;
			
		case BEAIM_ESEARCH_RESULTS_CLOSED:
			windows->EmailSearchResultsWindowClosed();
			break;

		case BEAIM_NEXT_CHAT_WINDOW:
		case BEAIM_PREV_CHAT_WINDOW:
			windows->SwitchUserWindow( message );
			break;

		case BEAIM_RELOAD_PREF_SETTINGS:
			DoGlobalPrefs();
			break;
			
		case BEAIM_TOGGLE_DESKBAR_ICON:
			DoDeskbarIcon( message->FindBool("enabled") );
			break;

		// needs to be fixed for thread-safeness?
		case BEAIM_SET_MY_SCREEN_NAME:
			strcpy( MyScreenName, message->FindString("new_userid") );
			break;

		default:
			BApplication::MessageReceived( message );
	}
}

//---------------------------------------------------------------------

void BeAIMApplication::DoBeAIMStartupStuff() {

	deskbarIconInstalled = false;
	
	// Create the necessary global objects
	net = new AIMNetThread;
	windows = new WindowManager;
	prefs = new GPreferences;
	list = new PeopleList;
	comm = new ProtocolManager;
	sounds = new classSoundMaster;
}

//---------------------------------------------------------------------

void BeAIMApplication::DoBeAIMShutdownStuff() {

	// remove the tray icon
	if( deskbarIconInstalled )
		DoDeskbarIcon( false );

	// net will take care of deleting itself
	if( net )
		net->Logout();
	net = 0;
	if( comm )
		comm->Close();
	comm = 0;
	delete list;
	list = 0;
	delete windows;
	windows = 0;
	delete prefs;
	prefs = 0;
	delete sounds;
	sounds = 0;
}

//---------------------------------------------------------------------

void BeAIMApplication::DoDeskbarIcon( bool install ) {

	if( install )
		TrayIcon::AddTrayIcon();	
	else
		TrayIcon::RemoveTrayIcon();	
	
	deskbarIconInstalled = install;
}

//---------------------------------------------------------------------

void BeAIMApplication::AboutRequested() {

	Say( "BeAIM Beta 2.1 (the R4.5 friendly version)\n\nChurned out between exams by:\n   Greg Nichols (nicholsg@central.edu)\n\nCode and bug fixes contributed by:\n   Sean Heber\n   Paul Stadig\n\nFor updates, useless info, etc. check the BeAIM at:\n   http://www.legions.com/beos/aim\n\n...beware of the squirrels..." );
}

//---------------------------------------------------------------------

void BeAIMApplication::DoGlobalPrefs() {

	// do sound prefs
	bool soundEnabled = prefs->ReadBool("gm_ps", true);
	if( sounds->IsEnabled() != soundEnabled )
		sounds->SetEnabled( soundEnabled );

	// do deskbar icon prefs
	bool deskbarEnabled = prefs->ReadBool("gm_dk", true);
	if( deskbarEnabled != deskbarIconInstalled ) {
		DoDeskbarIcon( deskbarEnabled );	
	}
}

//---------------------------------------------------------------------