#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ListItem.h>
#include <SupportKit.h>
#include "PeopleData.h"
#include "BuddyList.h"
#include "MiscStuff.h"
#include "Say.h"
#include "constants.h"

//#define PLIST_DEBUG

//-------------------------------------------------------------------------

PeopleList::PeopleList() {
	display = NULL;
	setup = NULL;
	commitAdds = 0;
	
	strcpy( UserName, "default" );
		
	// make the locker
	lock = new BLocker( "people_lock" );
}

//-------------------------------------------------------------------------

PeopleList::~PeopleList() {

	lock->Lock();
	GroupInfo* group;
	
	// Save the list if it isn't empty
	if( !commitList.IsEmpty() )
		Save();
	
	// Empty the main list
	while( main.Pop(group) )
		delete group;

	// delete the BLocker
	lock->Unlock();
	delete lock;
}

//-------------------------------------------------------------------------

void PeopleList::LoadUserSettings( char* user ) {

	lock->Lock();

	// copy the user name
	strcpy( UserName, user );
	
	// read the contents from a config file
	Load();
	commitList.Clear();			// don't need to commit the first time
	commitAdds = 0;
	
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::SetDisplayList( BOutlineListView* list ) {

	lock->Lock();
	display = list;
	GroupItem* groupItem;
	
	// Load all groups into the display
	for( unsigned int i = 0; i < main.Count(); ++i ) {
		groupItem = new GroupItem( main[i]->group, 0, main[i]->total );
		main[i]->item = groupItem;
		display->AddItem( groupItem );
	}		
	
	lock->Unlock();	
}

//-------------------------------------------------------------------------

void PeopleList::SetSetupList( BOutlineListView* list ) {

	lock->Lock();
	setup = list;
	PeopleSetupItem* groupItem;
	PeopleSetupItem* personItem;
	GenList<PersonInfo*>* plist;
	
	// Load all the groups into the list
	for( unsigned int i = 0; i < main.Count(); ++i ) {
	
		groupItem = new PeopleSetupItem( 0, main[i]->group, true );
		main[i]->setupitem = groupItem;
		setup->AddItem( groupItem );
		
		// Now load in all the people in that group (backwards, to maintain order)
		plist = &(main[i]->names);
		for( unsigned int j = (*plist).Count(); j > 0; --j ) {
			personItem = new PeopleSetupItem( (*plist)[j-1]->proto, (*plist)[j-1]->name, false );
			(*plist)[j-1]->setupitem = personItem;
			setup->AddUnder( personItem, groupItem );
		}
	}

	lock->Unlock();
}

//-------------------------------------------------------------------------

PersonInfo* PeopleList::AddPerson( unsigned int proto, char* person, char* group, bool commitNow, bool check )
{
	lock->Lock();
	GroupInfo* theGroup;
	PersonInfo* pi;
	
	// if we are supposed to check if the person is already there, do it
	if( check ) {
		if( FindPerson( proto, person ) ) {
			lock->Unlock();
			return NULL;		
		}		
	}
	
	// Whew. Done. Now find the group, and make it if it isn't there.
	theGroup = FindGroup( group, true );
	
	#ifdef PLIST_DEBUG
	printf( "adding person %s in group %s (proto %u)\n", person, group, proto );
	#endif
	
	// Set some information
	pi = new PersonInfo;
	pi->item = NULL;
	pi->setupitem = NULL;
	pi->parent = NULL;
	pi->proto = proto;
	pi->status = BS_OFFLINE;
	strcpy( pi->name, person );

	// Attach it to the list
	Attach( pi, theGroup );
	
	// add this person to the commit list
	AddCommitListItem( proto, person, true );
	
	// if we're supposed to commit right away, do that
	if( commitNow )
		Commit();
	
	lock->Unlock();
	return pi;
}

//-------------------------------------------------------------------------

void PeopleList::RemovePerson( unsigned int proto, char* person, bool commitNow ) {

	lock->Lock();
	PersonInfo* pi;
	unsigned int groupIndex = 0;
	
	// Find the person in the list, and if successful, remove it
	pi = FindPerson( proto, person, &groupIndex );
	if( pi )
		RemovePerson( pi, groupIndex, commitNow );
	
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::RemovePerson( PersonInfo* person, unsigned int groupIndex, bool commitNow ) {

	lock->Lock();
	
	#ifdef PLIST_DEBUG
	printf( "removing person %s...\n", person->name );
	#endif
	
	// detach the person from the list
	Detach( person, groupIndex );
	
	// Add the person to the commit list
	AddCommitListItem( person->proto, person->name, false );
	
	// if we're supposed to commit right away, do that
	if( commitNow )
		Commit();	
	
	// delete the sucker
	delete person;
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::SetPersonStatus( unsigned int proto, char* person, unsigned int status, unsigned int changeTo )
{
	lock->Lock();
	PersonInfo* setPerson;
	
	// Try and find that person and leave if it fails
	setPerson = FindPerson( proto, person );
	if( !setPerson ) {
		#ifdef PLIST_DEBUG
		printf( "person %u:%s not found! leaving.\n", proto, person );
		#endif
		lock->Unlock();
		return;
	}

	// call the function that does the real work
	SetPersonStatus( setPerson, status, changeTo );	
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::SetPersonStatus( PersonInfo* person, unsigned int status, unsigned int changeTo )
{
	lock->Lock();

	#ifdef PLIST_DEBUG
	printf( "changing status of %s\n", person->name );
	#endif

	// Put them in the "online" list, if needed	
	if( !isOnline(person->status) && isOnline( status ) ) {
		SetOnline( person, status );
		changeTo = status;
		sounds->PlaySound( WS_ENTER );
	}
	
	// Take them OFF the "online" list, if needed
	else if( isOnline(person->status) && !isOnline( status ) ) {
		person->item->SetStatus( BS_LEAVING );
		display->InvalidateItem( display->IndexOf(person->item) );
		changeTo = BS_BYEBYE;
		sounds->PlaySound( WS_EXIT );
	}
	
	// they ready to finally leave now?
	else if( status == BS_BYEBYE ) {
		SetOffline( person );
		status = person->status;		// don't actually change the status
	}
	
	// update the display
	else if( display && person->item ) {
		person->item->SetStatus( status );
		display->InvalidateItem( display->IndexOf(person->item) );
	}	
	
	// If this is going to require a change, set that up
	if( changeTo )
		SetupChange( person, changeTo );
		
	// now actually set their status
	person->status = status;		
	
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::ChangePerson( unsigned int oldproto, char* oldperson,
    						   unsigned int newproto, char* newperson, char* newgroup )
{
	lock->Lock();
	PersonInfo* oldobject;
	unsigned int groupIndex;
	
	// Find the old person's object
	oldobject = FindPerson( oldproto, oldperson, &groupIndex );
	if( !oldobject ) {
		lock->Unlock();
		return;
	}
	
	// Call the function that does the real work
	ChangePerson( oldobject, groupIndex, newproto, newperson, newgroup );
	
	lock->Unlock();		
}

//-------------------------------------------------------------------------

void PeopleList::ChangePerson( PersonInfo* oldperson, unsigned int groupIndex, unsigned int newproto,
											char* newperson, char* newgroup )
{
	#ifdef PLIST_DEBUG
	printf( "changing %s:%u (%s) to %s:%u (%s)\n", oldperson->name, oldperson->proto, oldperson->parent->group, newperson, newproto, newgroup );
	#endif

	lock->Lock();
	bool onlyGroupChange = false;
	PeopleSetupItem* removeSetupItem;
	PeopleItem* removeItem;

	// Get the display items	
	removeItem = oldperson->item;
	removeSetupItem = oldperson->setupitem;
	
	// Check to see what, if anything, has changed
	//if( strcasecmp(oldperson->name, newperson) == 0 && oldperson->proto == newproto ) {
	if( CompareScreenNames(oldperson->name, newperson) && oldperson->proto == newproto ) {
	
		// if nothing has changed, we can return
		if( strcasecmp( oldperson->parent->group, newgroup ) == 0 ) {
			lock->Unlock();
			return;
		}		
		// ONLY the group has changed... we can skip a bunch of steps
		else onlyGroupChange = true;
	}
	
	// change the old name in the list
	if( !onlyGroupChange ) {
	
		// check to see if the person is already there
		if( FindPerson( newproto, newperson ) ) {
			#ifdef PLIST_DEBUG
			printf( "That person is already there!\n" );
			#endif
			
			Say( "That person is already there!" );
			lock->Unlock();
			return;
		}
		
		// Remove the old person, and add the new one
		RemovePerson( oldperson, groupIndex );
		AddPerson( newproto, newperson, newgroup );
	}
	
	// Only the group was changed; everything else stays the same
	else {
	
		// Get the group item
		GroupInfo* groupItem = FindGroup( newgroup );
		if( !groupItem ) {
			lock->Unlock();
			return;
		}

		// Move the sucker
		Detach( oldperson, groupIndex );
		Attach( oldperson, groupItem );

		// correct the group numbers and display stuff
		if( setup && removeSetupItem && groupItem->setupitem )
			setup->MoveItem( setup->IndexOf(removeSetupItem), setup->IndexOf(groupItem->setupitem) + 1 );
	}
	
	lock->Unlock();		
}


//-------------------------------------------------------------------------

void PeopleList::CorrectGroupNumbers( GroupInfo* group, unsigned int status, bool add ) {

	// adjust the group's numbers
	if( add ) {
		group->total++;
		if( isOnline(status) )
			group->on++;
	} else {
		group->total--;
		if( isOnline(status) )
			group->on--;
	}
		
	// adjust the display, if needed
	if( display && group->item ) {
		group->item->SetNumbers( group->on, group->total );
		display->InvalidateItem( display->IndexOf(group->item) );
	}
}

//-------------------------------------------------------------------------

// groupindex, if not null, returns the index of the person within the group
PersonInfo* PeopleList::FindPerson( unsigned int proto, char* person, unsigned int* groupIndex )
{	
	lock->Lock();
	PersonInfo* pi = NULL;
	GenList<PersonInfo*>* plist;
	if( groupIndex )
		*groupIndex = 0;
	
	// Find the person... could be in any group... sigh
	for( unsigned int i = 0; i < main.Count(); ++i ) {
		plist = &(main[i]->names);
		for( unsigned int j = 0; j < plist->Count(); ++j ) {
			//if( strcasecmp( (*plist)[j]->name, person ) == 0 && 
			if( CompareScreenNames( (*plist)[j]->name, person ) && 
									(*plist)[j]->proto == proto ) {
				pi = (*plist)[j];
				if( groupIndex )
					*groupIndex = j;
				break;
			}			
		}
	}
	
	lock->Unlock();
	return pi;
}

//-------------------------------------------------------------------------

GroupInfo* PeopleList::FindGroup( char* group, bool make, unsigned int* index )
{	
	GroupInfo* target = NULL;
	lock->Lock();
	
	// Look through all the groups
	for( unsigned int i = 0; i < main.Count(); ++i )
		if( strcasecmp( main[i]->group, group ) == 0 ) {
			target = main[i];
			if( index )
				*index = i;
			break;
		}
			
	// If it's not there, make it (if we're supposed to)
	if( !target && make )
		target = AddGroup( group );

	lock->Unlock();
	return target;	
}

//-------------------------------------------------------------------------

void PeopleList::AddCommitListItem( unsigned int proto, char* name, bool add ) {	

	lock->Lock();
	
	// make a person capsule
	PersonCapsule added;
	strcpy( added.name, name );
	added.proto = proto;
	added.add = add;

	#ifdef PLIST_DEBUG	
	printf( "attempting to %s person %s:%u %s commit list\n", add?"add":"remove", name, proto, add?"to":"from" );
	#endif
	
	// Go through and look for a duplicate
	for( unsigned int i = 0; i < commitList.Count(); ++i ) {
	
		// whoa... found one!
		//if( strcasecmp(name, commitList[i].name) == 0 && commitList[i].proto == proto ) {
		if( CompareScreenNames(name, commitList[i].name) && commitList[i].proto == proto ) {

			// if it's not the same op, delete it... add/remove cancel each other out
			if( commitList[i].add != add ) {
				if( commitList[i].add )
					--commitAdds;
				commitList.Delete(i);				
			}
			
			// Otherwise, ignore it. No real point in adding/removing a name twice.
			lock->Unlock();
			return;
		}				
	}

	// should be ok to add it at this point... do so
	// adds go at the front of the list... removes go at the back
	if( add ) {
		commitList.Insert( added, 0 );
		++commitAdds;
	} else
		commitList.Push( added );

	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::DispatchCommitList( unsigned int proto, GenList<PersonCapsule>& sendList, bool add ) {

	lock->Lock();
	char* sendString = new char[ sendList.Count() * (INTERNAL_NAME_MAX+1) ];
	char Tab[] = " ";
	sendString[0] = '\0';
	BMessage* commitMsg;
	Tab[0] = char(9);
	
	// add everyone in the list onto the send string
	for( unsigned int i = 0; i < sendList.Count(); ++i ) {
		if( i != 0 )
			strcat( sendString, Tab );
		strcat( sendString, sendList[i].name );	
	}
	
	// make and send a BMessage for the commit stuff
	commitMsg = new BMessage(BEAIM_PEOPLELIST_COMMIT);
	commitMsg->AddBool( "add", add );
	commitMsg->AddString( "list", sendString );
	PostAppMessage( commitMsg );
	
	printf( "  dispatching %s list for protocol %u\n", add?"add":"remove", proto );
	printf( "---->|%s|\n\n", sendString );
	
	delete sendString;	
	lock->Unlock();
}

//-------------------------------------------------------------------------

// commits the changes... a rather inefficient function, so it's fortunate it doesn't
// need to be called all that often.
void PeopleList::Commit()
{
	lock->Lock();
	GenList<unsigned int> protoList;
	GenList<PersonCapsule> committed;
	unsigned int test;
	bool protoFound;

	// Save the list
	Save();	
	
	// Go through and make a list of all the protocols represented
	for( unsigned int i = 0; i < commitList.Count(); ++i ) {
		test = commitList[i].proto;
		protoFound = false;
		for( unsigned int j = 0; j < protoList.Count(); ++j ) {
			if( protoList[j] == test ) {
				protoFound = true;
				break;
			}
		}
		if( !protoFound )
			protoList.Add( test );
	}

	#ifdef PLIST_DEBUG
	printf( "total commit list size: %u\n", commitList.Count() );
	printf( "commit list adds: %u\n", commitAdds );
	printf( "commit list removes: %u\n", commitList.Count() - commitAdds );
	#endif

	// We now have a valid protocol list. Make the lists for each protocol.
	for( unsigned int i = 0; i < protoList.Count(); ++i ) {
		test = protoList[i];

		#ifdef PLIST_DEBUG
		printf( "%u: \n", test );
		#endif
		
		// Make an add list for that protocol
		committed.Clear();
		for( unsigned int j = 0; j < commitAdds; ++j ) {
			if( commitList[j].proto == test ) {
				#ifdef PLIST_DEBUG
				printf( "  add: %s\n", commitList[j].name );
				#endif
				committed.Add( commitList[j] );
			}
		}
		if( committed.Count() )
			DispatchCommitList( test, committed, true );

		// Make a remove list for that protocol
		committed.Clear();
		for( unsigned int j = commitAdds; j < commitList.Count(); ++j ) {
			if( commitList[j].proto == test ) {
				#ifdef PLIST_DEBUG
				printf( "  remove: %s\n", commitList[j].name );
				#endif
				committed.Add( commitList[j] );
			}
		}
		if( committed.Count() )
			DispatchCommitList( test, committed, false );	
	}

	// reset the commit list variables
	commitAdds = 0;
	commitList.Clear();
	lock->Unlock();	
}

//-------------------------------------------------------------------------

void PeopleList::SetOnline( unsigned int proto, char* person, unsigned int status ) {

	lock->Lock();
	PersonInfo* thePerson = FindPerson( proto, person );
	if( thePerson )
		SetOnline( thePerson, status );
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::SetOnline( PersonInfo* person, unsigned int status ) {

	lock->Lock();
	GroupInfo* group = person->parent;
	PeopleItem* newPerson;
	
	#ifdef PLIST_DEBUG
	printf( "setting online: %u:%s\n", person->proto, person->name );
	#endif
	
	// update the display
	if( display ) {
		if( person->item )
			person->item->SetStatus( status | BS_ENTERING );
		else {
			group->on++;
			newPerson = new PeopleItem( person->proto, person->name );
			newPerson->SetStatus( status | BS_ENTERING );
			display->AddUnder( newPerson, group->item );
			person->item = newPerson;
		}
		
		group->item->SetNumbers( group->on, group->total );
		display->InvalidateItem( display->IndexOf(group->item) );		
	}
	
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::SetOffline( unsigned int proto, char* person ) {

	lock->Lock();
	PersonInfo* thePerson = FindPerson( proto, person );
	if( thePerson )
		SetOffline( thePerson );
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::SetOffline( PersonInfo* person ) {

	lock->Lock();
	GroupInfo* group = person->parent;
	
	// Increment the online group count
	group->on--;
		
	// update the display
	if( display ) {
		group->item->SetNumbers( group->on, group->total );
		display->InvalidateItem( display->IndexOf(group->item) );
		display->RemoveItem( person->item );
		person->item = NULL;
	}
	
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::GetGroupInfo( char* group, unsigned int& online, unsigned int& total ) {

	lock->Lock();
	total = 0;
	online = 0;
	
	// Get the group and return if not there
	GroupInfo* gp = FindGroup( group );
	if( !gp ) {
		lock->Unlock();
		return;
	}
	
	total = gp->total;
	online = gp->on;
	
	#ifdef PLIST_DEBUG
	printf( "%s: (%u/%u)\n", group, online, total );
	#endif
	
	lock->Unlock();	
}

//-------------------------------------------------------------------------

void PeopleList::ChangeGroup( char* oldname, char* newname ) {
			
	lock->Lock();
	unsigned int groupPos;
	GroupInfo* group;
	PeopleSetupItem* psi;
	GroupItem* gi;
	
	// Find the group
	group = FindGroup( oldname, &groupPos );
	if( !group ) {
		lock->Unlock();
		return;
	}
	
	// Check to see if the new group already exists
	if( FindGroup( newname ) ) {
		#ifdef PLIST_DEBUG
		printf( "group already exists!\n" );
		#endif
		
		Say( "That group already exists!" );
		lock->Unlock();
		return;
	}
	
	// change the name
	strcpy(	group->group, newname );
	psi = group->setupitem;
	gi = group->item;
		
	// update the displays
	if( setup && psi ) {
		psi->SetInfo( 0, newname );
		setup->InvalidateItem( setup->IndexOf( psi ) );
	}
	if( display && gi ) {
		gi->SetGroupName( newname );
		display->InvalidateItem( display->IndexOf( gi ) );
	}
	
	lock->Unlock();			
}

//-------------------------------------------------------------------------

unsigned int PeopleList::CountPeople() {
//	return main.Count();

	lock->Lock();
	unsigned int count = 0;
	
	for( unsigned int i = 0; i < main.Count(); ++i )
		count += main[i]->names.Count();
		
	lock->Unlock();
	return count;
}

//-------------------------------------------------------------------------		
		
void PeopleList::GetPersonInfo( unsigned int proto, char* person, PersonInfo*& ret ) {

	lock->Lock();
	ret = FindPerson( proto, person );
	lock->Unlock();	
}
		
//-------------------------------------------------------------------------

void PeopleList::Load() {
	
	char tmpchar = 0;
	unsigned int i = 0, j = 0, k = 0, proto;
	char data[512];
	char person[128];
	char group[128];
	char strproto[128];
	bool isgroup;
	char filename[1024];
	
	// make the filename based on the current username
	MakeFilenameFromUsername( filename, UserName );
	
	// open the people file
	ptr = fopen( filename, "r" );
	if( ptr == 0 )
		return;

	// read through the lines in the file
	tmpchar = getc( ptr );
	while( tmpchar != -1 ) {
	
		i = 0;
		while( tmpchar != -1 ) {
			if( tmpchar == '\n' ) {
				data[i] = '\0';
				break;
			}
			data[i++] =	tmpchar;
			tmpchar = getc( ptr );
		}
		
		tmpchar = getc( ptr );
		k = 0;
		j = 0;
		
		// check: was this a group and not a person?
		isgroup = false;
		if( data[j++] == 'g' )
			isgroup = true;	

		// get the name
		while( data[j] && data[j] != char(9) )
			person[k++] = data[j++];
		person[k] = '\0';
		
		// if it was a group, add it and go on
		if( isgroup ) {
			AddGroup( person );
			continue;
		}
		
		// get the group
		k = 0; 	++j;
		while( data[j] && data[j] != char(9) )
			group[k++] = data[j++];
		group[k] = '\0';
		
		// get the protocol (string version)
		k = 0; 	++j;
		while( data[j] && data[j] != char(9) )
			strproto[k++] = data[j++];
		strproto[k] = '\0';
		proto = 42;
		
		// Add the new person
		AddPerson( proto, person, group );
	}
	
	// close the file output
	fclose( ptr );
}

//-------------------------------------------------------------------------

void PeopleList::Save() {

	lock->Lock();
	char filename[255];
	GenList<PersonInfo*>* plist;
	
	// make the filename based on the current username
	MakeFilenameFromUsername( filename, UserName );
	
	// open the people file
	ptr = fopen( filename, "w" );
	if( ptr == 0 )
		return;	
		
	// write out the entries
	for( unsigned int i = 0; i < main.Count(); ++i ) {
	
		// write out the group
		fprintf( ptr, "g%s\n", main[i]->group );
		
		// write out the entries in that group
		plist = &(main[i]->names);
		for( unsigned j = 0; j < (*plist).Count(); ++j )
			fprintf( ptr, "p%s%c%s%c%s\n", (*plist)[j]->name, 9, main[i]->group, 9, "AIMtv.01" );
	}
	
	fclose( ptr );
	lock->Unlock();	
}

//-------------------------------------------------------------------------

void PeopleList::GetListForProtocol( unsigned int proto, char*& list ) {

	lock->Lock();
	unsigned int count = 0;
	GenList<PersonInfo*>* plist;
	char tab[] = " ";
	tab[0] = 9;
	
	// Go through and count the number of people for this protocol
	for( unsigned int i = 0; i < main.Count(); ++i ) {
		plist = &(main[i]->names);
		for( unsigned int j = 0; j < (*plist).Count(); ++j )
			if( (*plist)[j]->proto == proto )
				++count;
	}
	
	#ifdef PLIST_DEBUG
	printf( "list proto count: %u\n", count );
	#endif
	
	if( !count ) {
		list = NULL;
		lock->Unlock();
		return;
	}
	
	// number of names needed + a tab after each name... caller must delete list!
	list = new char[ count * (INTERNAL_NAME_MAX+1) ];
	list[0] = '\0';
	
	// Go through and get the names for this protocol
	count = 0;
	for( unsigned int i = 0; i < main.Count(); ++i ) {
		plist = &(main[i]->names);
		for( unsigned int j = 0; j < (*plist).Count(); ++j )
			if( (*plist)[j]->proto == proto ) {
				if( count++ )
					strcat( list, tab );
				strcat( list, (*plist)[j]->name );
			}
	}	
	
	lock->Unlock();
}

//-------------------------------------------------------------------------

GroupInfo* PeopleList::AddGroup( char* group, bool check ) {

	lock->Lock();
	GroupItem* groupItem;
	GroupInfo* newGroup;
	PeopleSetupItem* groupSetupItem;
	
	// if we are supposed to, check and see if this group already exists
	if( check && FindGroup( group, false ) ) {
		lock->Unlock();
		return NULL;
	}
		
	#ifdef PLIST_DEBUG
	printf( "---> adding group %s\n", group );
	#endif
		
	// OK, it doesn't exist. Add it.
	newGroup = new GroupInfo;
	newGroup->on = newGroup->total = 0;
	newGroup->item = NULL;
	newGroup->setupitem = NULL;
	strcpy( newGroup->group, group );
	
	// Add it to the displays
	if( setup && display ) {
		groupItem = new GroupItem( group, 0, 0 );
		groupSetupItem = new PeopleSetupItem( 0, group, true );
		newGroup->item = groupItem;
		newGroup->setupitem = groupSetupItem;
		display->AddItem( groupItem );
		setup->AddItem( groupSetupItem );
	}

	// add it
	main.Add( newGroup );
	
	lock->Unlock();
	return newGroup;
}

//-------------------------------------------------------------------------

void PeopleList::RemoveGroup( char* group ) {
		
	lock->Lock();
	unsigned int index = 0;
	GroupInfo* grp;
	
	// Find the group object and its index
	grp = FindGroup( group, false, &index );

	#ifdef PLIST_DEBUG
	printf( "RemoveGroup: %s index: %u\n", group, index );
	#endif
	
	// Remove all the entries in the group
	for( unsigned int i = grp->names.Count(); i > 0; --i )
		RemovePerson( grp->names[i-1], i-1 );

	// Remove the group items from the displays
	if( display && grp->item )
		display->RemoveItem( grp->item );
	if( setup && grp->setupitem )
		setup->RemoveItem( grp->setupitem );		

	// Remove the group itself
	main.Delete( index );
	
	lock->Unlock();			
}

//-------------------------------------------------------------------------

unsigned int PeopleList::CountGroups() {
	
	lock->Lock();
	unsigned int count = main.Count();
	lock->Unlock();
	return count;
}
		
//-------------------------------------------------------------------------

void PeopleList::GetGroup( unsigned int i, char* group ) {

	lock->Lock();
	strcpy( group, main[i]->group );
	lock->Unlock();	
}

//-------------------------------------------------------------------------

void PeopleList::Debug() {
	
/*	lock->Lock();
	#ifdef PLIST_DEBUG
	printf( "\n\n" );
	#endif
	GenList<PersonInfo*>* plist;
	char debug[512];
	
	for( unsigned int i = 0; i < main.Count(); ++i ) {
		
		plist = &(main[i]->names);
		for( unsigned int j = 0; j < (*plist).Count(); ++j ) {
			sprintf( debug, "%s: %s (%u)", main[i]->group, (*plist)[j]->name, (*plist)[j]->proto );
			#ifdef PLIST_DEBUG
			printf( "%s\n", debug );
			#endif
		}
	}
	
	lock->Unlock();	*/
}

//-------------------------------------------------------------------------

void PeopleList::UpdateChangeList() {

	lock->Lock();

	// go through backwards to avoid corrupting array indices
	for( int32 i = changeList.Count() - 1; i >= 0; i-- ) {
	
		// decrement the time counter
		changeList[i].Counter--;

		if( changeList[i].Counter == 0 ) {
				
			// If the new status is BS_BYEBYE, it means to remove the buddy
			if( changeList[i].ChangeTo == BS_BYEBYE )
				display->RemoveItem( changeList[i].person->item );
			
			// Set the person's status
			SetPersonStatus( changeList[i].person, changeList[i].ChangeTo );

			// Delete the change from the changelist
			changeList.Delete(i);			
		}
	}
		
	lock->Unlock();	
}

//-------------------------------------------------------------------------

void PeopleList::SetupChange( unsigned int proto, char* person, unsigned int changeTo ) {

	lock->Lock();
	PersonInfo* thePerson = FindPerson( proto, person );
	if( thePerson )
		SetupChange( thePerson, changeTo );
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::SetupChange( PersonInfo* person, unsigned int changeTo ) {

	// Set up a new changeList entry
	lock->Lock();
	PersonChange change;
	change.ChangeTo = changeTo;
	change.Counter = BUDDY_CHANGE_TIME;
	change.person = person;
 
	// Go through and make sure that there isn't already a change record set up
	for( unsigned int i = 0; i < changeList.Count(); ++i ) {

		// if there is, change THAT and return
		if( changeList[i].person == person ) {
			changeList[i] = change;
			lock->Unlock();
			return;
		}
	}

	// OK, no existing change record... so add a new one
	changeList.Add( change );	
	lock->Unlock();	
}

//-------------------------------------------------------------------------

bool PeopleList::isOnline( const uint32 status ) {

	lock->Lock();
	bool ret;
	
	switch( status ) {
	
		case BS_OFFLINE:
		case BS_LEAVING:
		case BS_BYEBYE:
			ret = false;
			break;
			
		default:
			ret = true;
			break;	
	}
	
	lock->Unlock();
	return ret;
}

//-------------------------------------------------------------------------

void PeopleList::Attach( PersonInfo* newPerson, char* group, bool make ) {

	lock->Lock();
	GroupInfo* theGroup = FindGroup( group, make );
	Attach( newPerson, theGroup );
	lock->Unlock();
}

//-------------------------------------------------------------------------

void PeopleList::Attach( PersonInfo* person, GroupInfo* group ) {

	lock->Lock();
	PeopleSetupItem* psi;
	
	#ifdef PLIST_DEBUG
	printf( "attaching %s to group %s\n", person->name, group->group );
	#endif

	// Add it to the list
	group->names.Add( person );
	person->parent = group;
	
	// Add it to the setup display
	if( setup && group->setupitem ) {
		psi = new PeopleSetupItem( person->proto, person->name, false );
		setup->AddUnder( psi, group->setupitem );
		person->setupitem = psi;
	}	

	// set the new group stuff
	CorrectGroupNumbers( group, person->status, true );
	
	lock->Unlock();
}
		
//-------------------------------------------------------------------------
		
void PeopleList::Detach( PersonInfo* person, unsigned groupIndex ) {

	lock->Lock();
	GroupInfo* group = person->parent;
	
	#ifdef PLIST_DEBUG
	printf( "detaching %s from group %s\n", person->name, person->parent->group );
	#endif

	// remove the person from the list
	group->names.Delete( groupIndex );
	person->parent = NULL;
	CorrectGroupNumbers( group, person->status, false );	
	
	// Remove it from the setup display
	if( setup && person->setupitem )
		setup->RemoveItem( person->setupitem );
		
	// Remove it from the online display
	if( display && person->item )
		display->RemoveItem( person->item );

	lock->Unlock();
}

//=========================================================================

PeopleSetupItem::PeopleSetupItem( unsigned int pr, char *s, bool group ) {

	strcpy( person, s );
	proto = pr;
	isgroup = group;
	
	comm->TranslatePersonToDisplay( proto, person, displayname );
}

//-------------------------------------------------------------------------

PeopleSetupItem::~PeopleSetupItem() {
}

//-------------------------------------------------------------------------

char* PeopleSetupItem::GetPersonName() {
	return person;
}

//-------------------------------------------------------------------------

char* PeopleSetupItem::GetDisplayName() {
	return displayname;
}

//-------------------------------------------------------------------------

unsigned int PeopleSetupItem::GetProtocol() {
	return proto;
}

//-------------------------------------------------------------------------

void PeopleSetupItem::DrawItem( BView *owner, BRect frame, bool complete ) {

	rgb_color color;

	// Make the selection color (shouldn't be hardcoded!)
	rgb_color kHighlight; 
	kHighlight.red = kHighlight.blue = 222;
	kHighlight.green = 219;
	
	// Grab the owner's font, put it into boldface if it's expanded
	BFont baseFont;
	owner->GetFont( &baseFont );
	BFont ownerFont( baseFont );
	
	// does the background need to be redrawn?
	if( IsSelected() || complete ) {
		
		// pick the appropriate background color
        if( IsSelected() ) {
        	color = kHighlight;
        	owner->SetLowColor( 222, 219, 222 );
        } else {
			color = owner->ViewColor();
			owner->SetLowColor( 255, 255, 255 );
		}
	
		// draw the background		
		owner->SetHighColor(color);
		owner->FillRect(frame);
	}

	// set the font and draw the string
	owner->SetHighColor(0,0,0);
	owner->SetFont( &ownerFont );
	owner->MovePenTo(frame.left + 4, frame.bottom - 2);
	owner->DrawString( displayname ); 
	
	// reset the font and colors
	owner->SetFont( &baseFont );
	owner->SetLowColor( 255, 255, 255 );
}

//-------------------------------------------------------------------------

void PeopleSetupItem::SetInfo( unsigned int pr, char *s ) {

	strcpy( person, s );
	proto = pr;
	
	comm->TranslatePersonToDisplay( proto, person, displayname );
}

//=========================================================================

GroupItem::GroupItem( char *s, unsigned int on, unsigned int tot ) {
	
	name = (char*)malloc( strlen(s) + 1 );
	strcpy( name, s );
	online = on;
	total = tot;
}

//-------------------------------------------------------------------------

GroupItem::~GroupItem() {
	free(name);
}

//-------------------------------------------------------------------------

char* GroupItem::GetGroupName() {
	return name;
}

//-------------------------------------------------------------------------

void GroupItem::DrawItem( BView *owner, BRect frame, bool complete ) {

	rgb_color color;
	
	// Make a string with the right stuff in it
	char* GroupTitle = (char*)malloc( strlen(name) + 25 );
	sprintf( GroupTitle, "%s (%u/%u)", name, online, total );

	// Make the selection color (shouldn't be hardcoded!)
	rgb_color kHighlight; 
	kHighlight.red = kHighlight.blue = 222;
	kHighlight.green = 219;
	
	// Grab the owner's font, put it into boldface if it's expanded
	BFont baseFont;
	owner->GetFont( &baseFont );
	BFont ownerFont( baseFont );
	if( IsExpanded() )
		ownerFont.SetFace( B_BOLD_FACE );
	
	// does the background need to be redrawn?
	if( IsSelected() || complete ) {
		
		// pick the appropriate background color
        if( IsSelected() ) {
        	color = kHighlight;
        	owner->SetLowColor( 222, 219, 222 );
        } else {
			color = owner->ViewColor();
			owner->SetLowColor( 255, 255, 255 );
		}
	
		// draw the background		
		owner->SetHighColor(color);
		owner->FillRect(frame);
	}

	// set the font and draw the string
	owner->SetHighColor(0,0,0);
	owner->SetFont( &ownerFont );
	owner->MovePenTo(frame.left + 4, frame.bottom - 2);
	owner->DrawString( GroupTitle ); 
	
	// reset the font and colors
	owner->SetFont( &baseFont );
	owner->SetLowColor( 255, 255, 255 );
}

//-------------------------------------------------------------------------

void GroupItem::SetGroupName( char* s ) {

	free( name );
	name = (char*)malloc( strlen(s) + 1 );
	strcpy( name, s );
}

//-------------------------------------------------------------------------

void GroupItem::SetNumbers( unsigned int on, unsigned int tot ) {

	online = on;
	total = tot;
}

//=========================================================================

PeopleItem::PeopleItem( unsigned int pr, char *s, int32 stat ) {

	proto = pr;
	status = stat;

	strcpy( person, s );
	comm->TranslatePersonToDisplay( proto, person, displayname );
}

//-------------------------------------------------------------------------

void PeopleItem::SetInfo( unsigned int pr, char *s ) {

	proto = pr;
	strcpy( person, s );
	comm->TranslatePersonToDisplay( proto, person, displayname );
}

//-------------------------------------------------------------------------

PeopleItem::~PeopleItem() {
}

//-------------------------------------------------------------------------

char* PeopleItem::GetPersonName() {
	return person;
}

//-------------------------------------------------------------------------

unsigned int PeopleItem::GetProto() {
	return proto;
}

//-------------------------------------------------------------------------

char* PeopleItem::GetDisplayName() {
	return displayname;
}

//-------------------------------------------------------------------------

void PeopleItem::SetStatus( unsigned int stat ) {
	status = stat;
}

//-------------------------------------------------------------------------

void PeopleItem::DrawItem( BView *owner, BRect frame, bool complete ) {

	rgb_color color;

	// Make the selection color (shouldn't be hardcoded!)
	rgb_color kHighlight; 
	kHighlight.red = kHighlight.blue = 222;
	kHighlight.green = 219;
	
	// Grab the owner's font, to be fiddled with if needed
	BFont baseFont;
	owner->GetFont( &baseFont );
	BFont ownerFont( baseFont );
	
	// does the background need to be redrawn?
	if( IsSelected() || complete ) {
		
		// pick the appropriate background color
        if( IsSelected() ) {
        	color = kHighlight;
        	owner->SetLowColor( 222, 219, 222 );
        } else {
			color = owner->ViewColor();
			owner->SetLowColor( 255, 255, 255 );
		}
	
		// draw the background		
		owner->SetHighColor(color);
		owner->FillRect(frame);
	}
	
	// Set colors and font attributes based on buddy status
	if( status & BS_ENTERING )
		ownerFont.SetFace( B_BOLD_FACE );
	if( status & BS_LEAVING )
		ownerFont.SetFace( B_ITALIC_FACE );
	if( status & BS_IDLE )
		owner->SetHighColor(105,106,105);
	else
		owner->SetHighColor(0,0,0);


	// set the font and draw the string
	owner->SetFont( &ownerFont );
	owner->MovePenTo(frame.left + 4, frame.bottom - 2);
	owner->DrawString( displayname );

	// reset the font and colors
	owner->SetFont( &baseFont );
	owner->SetLowColor( 255, 255, 255 );
	owner->SetHighColor( 0, 0, 0 );
}

//-------------------------------------------------------------------------