#ifndef _PEOPLE_DATA_H_
#define _PEOPLE_DATA_H_

#include <stdio.h>
#include <OutlineListView.h>
#include <ListItem.h>
#include "GenList.h"
#include "constants.h"

struct GroupInfo;
struct PersonInfo;

//=========================================================================

class PeopleSetupItem : public BListItem
{ 
	public: 
		PeopleSetupItem( unsigned int pr, char* s, bool group );
		~PeopleSetupItem(); 
		virtual void DrawItem( BView *owner, BRect frame, bool complete = false );
		char* GetPersonName();
		char* GetDisplayName();
		unsigned int GetProtocol();
		bool isGroup() { return isgroup; };
		void SetInfo( unsigned int pr, char* s );

	private: 
		char person[INTERNAL_NAME_MAX];
		char displayname[DISPLAY_NAME_MAX];
		unsigned int proto;
		bool isgroup;
};

//=========================================================================

class GroupItem : public BListItem
{ 
	public: 
		GroupItem( char *s, unsigned int on = 0, unsigned int tot = 0 );
		~GroupItem(); 
		virtual void DrawItem( BView *owner, BRect frame, bool complete = false ); 
		char* GetGroupName();
		void  SetGroupName( char* );		
		void SetNumbers( unsigned int on, unsigned int tot );

	private: 
		char* name; 
		unsigned int online, total;
};

//=========================================================================

class PeopleItem : public BListItem
{ 
	public: 
		PeopleItem( unsigned int pr, char *s, int32 stat = BS_ACTIVE );
		~PeopleItem(); 
		virtual void DrawItem( BView *owner, BRect frame, bool complete = false ); 
		char* GetPersonName();
		char* GetDisplayName();
		unsigned int GetProto();
		void SetStatus( unsigned int );
		void SetInfo( unsigned int pr, char* s );

	private: 
		char person[INTERNAL_NAME_MAX];
		char displayname[DISPLAY_NAME_MAX];
		unsigned int proto;
		unsigned int status;
};

//=========================================================================

struct PersonCapsule {
	unsigned int proto;
	char name[INTERNAL_NAME_MAX];
	bool add;
};

//=========================================================================

struct PersonInfo {
	PeopleItem* item;
	PeopleSetupItem* setupitem;
	GroupInfo* parent;
	unsigned int proto;
	char name[INTERNAL_NAME_MAX];
	unsigned int status;
	
	bool operator==( PersonInfo& p ) {
		if( (strcasecmp(p.name,name) == 0) && (proto == p.proto) )
			return true;
		return false;
	}
};

//=========================================================================


struct GroupInfo {
	
	// trash everything in the list
	~GroupInfo() {
		PersonInfo* del;
		while( names.Pop(del) )
			delete del;
	}

	GroupItem* item;
	PeopleSetupItem* setupitem;
	unsigned int on;
	unsigned int total;
	char group[DISPLAY_NAME_MAX];
	GenList<PersonInfo*> names;	
};

//=========================================================================

// structure to handle timed list updates
struct PersonChange {
	PersonInfo* person;
	unsigned int ChangeTo;
	unsigned int Counter;
};

//=========================================================================

class PeopleList {

	public:
		PeopleList();
		~PeopleList();
		void SetDisplayList( BOutlineListView* list );
		void SetSetupList( BOutlineListView* list );
		void LoadUserSettings( char* user );
		
		PersonInfo* AddPerson( unsigned int proto, char* person, char* group, bool commitnow=false, bool check=true );
		void SetOnline( unsigned int proto, char* person, unsigned int status );
		void SetOnline( PersonInfo*, unsigned int status );
		void SetOffline( unsigned int proto, char* person );
		void SetOffline( PersonInfo* );
		void RemovePerson( unsigned int proto, char* person, bool commitnow=false );
		void RemovePerson( PersonInfo*, unsigned int groupIndex, bool commitnow=false );
		PersonInfo* FindPerson( unsigned int proto, char* person, unsigned int* gi = NULL );	
		void ChangePerson( unsigned int oldproto, char* oldperson,
						   unsigned int newproto, char* newperson, char* group );
		void ChangePerson( PersonInfo*, unsigned int groupIndex, unsigned int newproto,
						   char* newperson, char* group );	
		void SetPersonStatus( unsigned int proto, char* person, unsigned int, unsigned int = 0 );
		void SetPersonStatus( PersonInfo*, unsigned int,  unsigned int = 0 );

		GroupInfo* FindGroup( char* group, bool make = false, unsigned int* gi = NULL );
		GroupInfo* AddGroup( char* group, bool check = true );
		void RemoveGroup( char* group );
		void ChangeGroup( char*, char* );
		unsigned int CountGroups();
		void GetGroup( unsigned int, char* );
		
		void AddCommitListItem( unsigned int proto, char* name, bool add );
		void DispatchCommitList( unsigned int proto, GenList<PersonCapsule>&, bool add );
		void Commit();
		void UpdateChangeList();
		
		void Attach( PersonInfo*, char*, bool make = false );
		void Attach( PersonInfo*, GroupInfo* );
		void Detach( PersonInfo*, unsigned groupIndex );
		
		void GetGroupInfo( char*, unsigned int&, unsigned int& );
		void GetListForProtocol( unsigned int proto, char*& list );
		
		unsigned int CountPeople();
		void GetPersonInfo( unsigned int proto, char* person, PersonInfo*& );
		
		void Debug();
		
		void SetUserName( char* username );
		
		void Load();
		void Save();
	
	private:
		
		void SetupChange( unsigned int proto, char* person, unsigned int changeTo );
		void SetupChange( PersonInfo* person, unsigned int changeTo );
		void CorrectGroupNumbers( GroupInfo*, unsigned int status, bool add );
		bool isOnline( const uint32 );
	
		char UserName[DISPLAY_NAME_MAX];
	
		BOutlineListView* display;
		BOutlineListView* setup;
		GenList<PersonCapsule> commitList;
		GenList<PersonChange> changeList;
		GenList<GroupInfo*> main;		
				
		FILE *ptr;
		BLocker* lock;
		unsigned int commitAdds;
};


#endif
