#ifndef _WINDOW_MANAGER_H_
#define _WINDOW_MANAGER_H_

#include <Window.h>
#include <Message.h>
#include <Locker.h>
#include "BuddyList.h"
#include "GenList.h"
#include "constants.h"
#include "LoginBox.h"
#include "DataSender.h"
#include "ResultsWindow.h"

// Used to define the list of open IM windows
struct IMWindowItem {
	IMWindowItem() { window = NULL; };
	IMWindowItem( unsigned int pr, char* sn, int tp, BWindow* w ) {
		strcpy( name, sn );
		window = w;
		proto = pr;
		type = tp;
	};
	char name[INTERNAL_NAME_MAX];
	unsigned int proto;
	int type;
	BWindow* window;
};


// WindowManager - the class that handles the opening and closing of various
//   BeAIM windows, like the IM Windows.
class WindowManager {

	public:
		WindowManager();
		
		void OpenBuddyList( BPoint* point = NULL );
		void OpenPrefsWindow( bool globalOnly = false, BPoint* point = NULL );
		void OpenSignOnWindow( BPoint* point = NULL );
		void CloseSignOnWindow( BPoint* point = NULL );
		void DisplayMessage( char* );
		void DisplayError( char* );
		void MakeDataSenderWindow();
		
		BWindow* OpenUserWindow( BMessage* message );
		void UserWindowOpened( BMessage* message );
		void UserWindowClosed( BMessage* message );
		void SwitchUserWindow( BMessage* message );
		void ForwardIncomingMessage( BMessage* message );
		void BroadcastMessage( BMessage* message );
		
		void ForwardEmailSearchResults( BMessage* message );
		void OpenEmailSearchResultsWindow( BMessage* message );
		void EmailSearchResultsWindowClosed();

		void SendBuddyListMessage( BMessage* message );
		void SendSignOnMessage( BMessage* message );
		
		void MakeDialogFrame( BRect&, BPoint );
		void MakeDialogFrame( BRect&, BWindow* );
		void CenterFrame( BRect& );
		
		void ToggleHidden();
		
		void OpenInputWindow( char* title, char* label, BMessage* = NULL, BWindow* window = NULL,
							  char* initial = NULL, int maxLen = 0, bool toApp = false, char* fname = "value" );

	private:
		void CalcNewIMRect();
		BWindow* MakeMessageWindow( unsigned int proto, char* actualName, char* displayName );
		BWindow* MakeInfoWindow( unsigned int proto, char* actualName, char* displayName, BWindow* posWindow = NULL );
	
		BRect IMWindowRect;
		GenList<IMWindowItem> IMWindows;
		BuddyList* bList;
		LoginWindow* signOn;
		SearchResultsWindow* resWin;
		BLocker* lock;
};

#endif
